<?php
/**
 * Class to work with content of an apa xml file.
 * It provides an opportunity to get necessary parameters of news
 *
 * @author Andrew Voloshin <andrew.voloshin@oberig.com>
 *
 */

class parserXML{

    /**
     * @var object object of SimpleXMLElement class
     */
    public $xml;

    /**
     * Constructor
     */
    public function __construct($xml){

        $this->xml = $xml;
    }
    /**
     * Function returns content of tag with name $field from META tag
     *
     * @param string $field name of tag which content should be returned
     *
     * @return string - content of tag
     */
    public function getMeta($field = null){

        if(isset($field)){
            return (string)$this->xml->HEAD->META[$field];
        }else{
            return $this->xml->HEAD->META;
        }
    }
    /**
     * Function returns content of tag with name $field from BODY tag
     *
     * @param string $field name of tag which content should be returned
     *
     * @return string - content of tag
     */
    public function getBody($field = null){

        if(isset($field)){
            return $this->xml->BODY->$field;
        }else{
            return $this->xml->BODY;
        }
    }
    /**
     * Function returns content of tag with name $field from HEAD tag
     *
     * @param string $field name of tag which content should be returned
     *
     * @return string - content of tag
     */
    public function getHead($field = null){

        if(isset($field)){
            return $this->xml->HEAD->$field;
        }else{
            return $this->xml->HEAD;
        }
    }
    /**
     * Function returns content of tag with name $field from INFO tag
     *
     * @param string $field name of tag which content should be returned
     *
     * @return string - content of tag
     */
    public function getInfo($field = null){

        if(isset($field)){
            return $this->xml->INFO->$field;
        }else{
            return $this->xml->INFO;
        }
    }
    /**
     * Function returns DOCID of current xml file
     *
     * @return string - docid of the document
     */
    public function getDocId(){

        return (string)$this->getMeta('DOCID');
    }
    /**
     * Function returns ID of current xml file
     *
     * @return string - id of the document
     */
    public function getApaId(){

        return (string)$this->getMeta('ID');
    }
    /**
     * Function returns string without accents
     *
     * @param string $string string in which should be removed accents
     *
     * @return string - string without accents
     */
    public static function unaccent($string){
        return strtr(utf8_decode($string),
            utf8_decode('ŠŒŽšœžŸ¥µÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝßàáâãäåæçèéêëìíîïðñòóôõöøùúûüýÿ'),
            'SOZsozYYuAAAAAAACEEEEIIIIDNOOOOOOUUUUYsaaaaaaaceeeeiiiionoooooouuuuyy');
    }
    /**
     * Function returns title of current xml file
     *
     * @return string - title of the document
     */
    public function getTitle(){

        return (string)$this->getHead('TITLE');
    }
    /**
     * Function returns date of current xml file
     *
     * @return string - date in unix form of the document
     */
    public function getDate(){

        $date = (string)$this->getMeta('TIMEDATE');
        //Migrate to normal format
        $date = date('Y-m-d G:i:s',strtotime($date));
        $date = new DateTime($date);
        return $date->format('U');
    }
    /**
     * Function returns DOCTYP of current xml file
     *
     * @return string - DOCTYP of the document
     */
    public function getDocType(){

        return (string)$this->getMeta('DOCTYP');
    }
    /**
     * Function returns array of child files
     *
     * @return array - child files array
     */
    public function getRelations(){

        $relations = array();
        foreach($this->getBody('TEXT')->LINK as $relation){
            $relations[] = (string)$relation['HREF'];
        }
        return $relations;
    }
    /**
     * Function returns array of child files parameters
     *
     * @return array child files parameters
     */
    public function getFiles(){

        $files = array();
        foreach($this->getBody('TEXT')->LINK as $file){
            $files[] = array(
                'href' => (string)$file['HREF'],
                'size' => (string)$file['SIZE'],
                'width' => (string)$file['WIDTH'],
                'height' => (string)$file['HEIGHT'],
                'format' => (string)$file['FORMAT'],
            );
        }
        return $files;
    }
    /**
     * Function returns place
     *
     * @return string - content of place tag
     */
    public function getPlace(){

        return (string)$this->getBody('PLACE');
    }
    /**
     * Function returns source
     *
     * @return string - content of source tag
     */
    public function getSource(){

        return (string)$this->getBody('SOURCE');
    }
    /**
     * Function returns author
     *
     * @return string - content of author tag
     */
    public function getAuthor(){

        return (string)$this->getInfo('AUTHOR');
    }
    /**
     * Function returns title of article in text tag
     *
     * @return string -  title of article
     */
    public function getTeaser(){

        return (string)$this->getBody('TEXT')->LEAD;
    }
    /**
     * Function returns title of document without accents and shortened to 64 characters
     *
     * @return string - document title
     */
    public function getFilename(){

        $filename = self::unaccent((string)$this->getHead('TITLE'));
        $filename = strtr($filename,array(' ' => '_', '"'=>''));
        $filename = substr($filename,0,64);
        return $filename;
    }
    /**
     * Function returns keywords of document
     *
     * @return array - keywords
     */
    public function getKeywords(){

        $keywords = array();
        foreach($this->getHead('KEYWORD') as $kw){
            $keywords[] = (string)$kw;
        }
        $keywords = implode(',',$keywords);

        return $keywords;
    }
    /**
     * Function returns text of article
     *
     * @return string - text
     */
    public function getText(){

        $text = array();
        foreach($this->getBody('TEXT')->P as $paragraph){
            $text[] = (string)$paragraph;
        }
        $text = '<p>'.implode('</p><p>',$text).'</p>';

        return $text;
    }
    /**
     * Function returns docid of news line which should be removed
     *
     * @return string - docid
     */
    public function getDeleteId(){

        return (string)$this->xml->delitem['apaid'];
    }

}