<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget, which renders the AJAX code that contacts the SuggestionsController
 * to get and display suggestions from the Marketing Automation tracker.
 */
class MarkaSuggestionsWidget extends BaseJuiWidget
{
    /**
     * The number of suggestions that should be retrieved.
     * @var int 
     */
    public $suggestionsCount;
    
    /**
     * The array of categories that all returned suggestions must have (they may
     * also have other categories as well).
     * This is optional and if set, it must be a list of strings, 
     * e.g. array('category1', 'category2')
     * @var array 
     */
    public $categories = NULL;
    
    /**
     * The ID of the page on which this widget is being included. This is not the CMS page ID,
     * but an ID that is only used for Marketing Automation. It must be a string
     * of the following format:
     * for pages without a channel ID: CMSPageID (e.g. '1234' or '4923')
     * for pages with a channel ID: 'c' . ChannelID . '.' . CMSPageID (e.g. 'c2.1234' or 'c8.4923')
     * 
     * This ID is optional. If set, it tells Marketing Automation to make sure 
     * that the returned suggestions do not include the current page. If not set,
     * the current page might also be included in the suggestions (if it is a 
     * tracked page) if the tracking request for this page arrives at the server
     * after the request for page suggestions (in this case Marketing Automation
     * does not know that the user has already seen this page) or if not enough
     * unseen pages are available and random pages are added to the suggestions
     * (the random pages do not consider the pages that the user has already seen).
     * @var string 
     */
    public $currentPageId = NULL;
    
    /**
     * The view that should be used to render the suggestions (this is not done
     * by this widget, but by the SuggestionsController, which is contacted by the AJAX
     * code rendered by this widget).
     * Default: "MarkaSuggestionsSimpleLinksList"
     * 
     * This view is passed the parameter suggestedPages, which is an array
     * of the suggested pages infos in the order they were provided by
     * Marketing Automation (sorted by descending view count). Each of these
     * objects is an associative array that contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL. If there are no
     * suggested pages, the parameter suggestedPages will be NULL and it is up 
     * to the view to either not render anything or to render some default links.
     * 
     * @var string 
     */
    public $view = 'MarkaSuggestionsSimpleLinksList';
    
    /**
     * The base URL of the SuggestionsController, which gets and renders the
     * suggestions and which is contacted by the AJAX code rendered by this widget.
     * @var string
     */
    private $suggestionsControllerBaseUrl = '/marketingautomation/suggestions/suggestions';
    
    /**
     * Renders the AJAX code, which contacts the SuggestionsController to get
     * the suggestions.
     * 
     * @return void
     */
    public function run()
    {
        if (empty($this->suggestionsCount)) {
            die('No suggestionsCount specified.');
        }
        $markaModule = Yii::app()->getModule('marketingautomation');
        if (!$markaModule->isMarkaSuggestionsEnabled()) {
            return;
        }
        $url = $this->suggestionsControllerBaseUrl . '?count=' . strval($this->suggestionsCount);
        // In this widget we use urlencode() instead of rawurlencode(), because the URL does not refer
        // to the marka-tracker, but to a PHP controller.
        $url .= '&view=' . urlencode($this->view);
        if (!empty($this->categories)) {
            $url .= '&categories=' . $this->_assembleCategoriesStr($this->categories);
        }
        if (!empty($this->currentPageId)) {
            $url .= '&currentPageId=' . urlencode($this->currentPageId);
        }
        $this->render('MarkaSuggestionsAjaxCode', array('url' => $url));
    }
    
    /**
     * Converts the specified categories array into a categories string for
     * the get suggestions REST request and returns that string. This string
     * is of the form category1;category2;category3;category4
     * @param array $categories
     * @return string
     */
    private function _assembleCategoriesStr(&$categories)
    {
        $ret = '';
        $first = true;
        foreach ($categories as $category) {
            if (!$first) {
                $ret .= ';';
            } else {
                $first = false;
            }
            $ret .= urlencode($category);
        }
        return $ret;
    }

}
