<?php

/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * This is the model class for table "{{personalization_attributes}}".
 * It represents single personalisation attribute which can be assigned to user.
 * In database stored hierarchical information about these attributes and they can be represented as tree.
 *
 * The followings are the available columns in table '{{personalization_attributes}}':
 * @property integer $id
 * @property integer $parent_id
 * @property string $name
 * @property string $title
 * @property string $path
 */
class PersonalisationAttribute extends CActiveRecord
{

    /**
     * @var array $_flatLists "inclass" cache for personalisation flat rules
     */
    private static $_flatLists = array();

    /**
     * Returns the static model of the specified AR class.
     *
     * @param string $className active record class name.
     *
     * @return PersonalisationAttribute the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * returnes a table name
     *
     * @return string the associated database table name
     */
    public function tableName()
    {
        return '{{personalisation_attributes}}';
    }

    /**
     * returns validation rules
     *
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('name, title', 'required'),
            array('name, title', 'common.components.NotContainsTagsValidator'),
            array('name', 'unique'),
            array('parent_id', 'numerical', 'integerOnly' => true),
            array('name, title', 'length', 'max' => 255),
            array('path', 'safe'),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, parent_id, name', 'safe', 'on' => 'search'),
        );
    }

    /**
     * sets id of parent
     *
     * @param \PersonalisationAttribute $parent
     *
     * @return void
     */
    public function childOf(PersonalisationAttribute $parent)
    {
        $this->parent_id = $parent->id;
        $this->path = empty($parent->path) ? '.' . $parent->id . '.' : $parent->path . $parent->id . '.';
    }

    /**
     * Find all children of current personalisation attribute
     *
     * @return array
     */
    public function getChildren()
    {
        return PersonalisationAttribute::model()->findAll("path LIKE '%.{$this->id}.%'");
    }

    /**
     * Get children ids
     *
     * @return array
     */
    public function getChildrenIds()
    {
        $data = Yii::app()->db->createCommand()
            ->select('id')
            ->from($this->tableName())
            ->where("path LIKE '%.{$this->id}.%'")
            ->queryAll(false);
        $ids = array_map(
            function($item)
            {
                return $item[0];
            }, $data
        );

        return $ids;
    }

    /**
     * returns relations rules
     *
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
            array(self::HAS_MANY, 'UserPersonalisationAttribute', 'personalisation_attribute_id')
        );
    }

    /**
     * Returns customized attribute labels (name=>label)
     *
     * @return array
     */
    public function attributeLabels()
    {
        return array(
            'id' => 'ID',
            'parent_id' => 'Parent',
            'name' => 'Name',
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     *
     * @return \CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;

        $criteria->compare('id', $this->id);
        $criteria->compare('parent_id', $this->parent_id);
        $criteria->compare('name', $this->name, true);

        return new CActiveDataProvider(
            $this,
            array(
                'criteria' => $criteria,
            )
        );
    }

    /**
     * Delete personalisation attribute with children and UserPersonalisationAttribute records
     *
     * @return bool
     */
    public function delete()
    {
        $transaction = Yii::app()->db->beginTransaction();
        try {
            $removeIds = $this->getChildrenIds();
            if (!empty($removeIds)) {
                PersonalisationAttribute::model()->deleteAll('id IN (' . implode(',', $removeIds) . ')');
            }
            $removeIds[] = $this->id;
            UserPersonalisationAttribute::model()->deleteAll('personalisation_attribute_id IN (' . implode(',', $removeIds) . ')');
            $result = parent::delete();
            $transaction->commit();
            return $result;
        } catch (Exception $e) {
            $transaction->rollback();
            return false;
        }
    }

    /**
     * Get personalisation attributes for user in flat list form
     *
     * @param int  $userId user id
     * @param bool $cache  defines if rrsult should be cached
     *
     * @return array
     */
    public static function flatListForUser($userId, $cache = true)
    {
        $flatList = array();
        if (!empty($userId)) {
            if (isset(self::$_flatLists[$userId])) {
                $flatList = self::$_flatLists[$userId];
                Yii::trace('Flatlist for userId='.$userId.' taken from inclass cache');
            } else {
                $userPersAttributes = UserPersonalisationAttribute::model()
                    ->cache($cache ? Yii::app()->getModule('personalisation')->cacheTime : 0)
                    ->with('personalisationAttribute')
                    ->findAllByAttributes(array('user_id' => $userId));
                if ($userPersAttributes) {
                    foreach ($userPersAttributes as $attr) {
                        $flatList[] = $attr->personalisationAttribute->name;
                    }
                }
                Yii::trace('Flatlist for userId='.$userId.' was put to inclass cache');
                self::$_flatLists[$userId] = $flatList;
            }
        }

        return $flatList;
    }

    /**
     * Get associative array which represents all existed personalisation attributes in tree structure
     *
     * @static
     *
     * @return array
     */
    public static function getTree()
    {
        $personalisationAttributes = PersonalisationAttribute::model()->findAll(
            array(
                'order'=>'path ASC,
                title ASC'
            )
        );
        $tree = array('id' => 0, 'item' => array());

        foreach ($personalisationAttributes as $attr) {
            $node = array(
                'id' => $attr->id,
                'text' => $attr->title,
                'userdata' => array(
                    array('name' => 'name', 'content' => $attr->name),
                    array('name' => 'title', 'content' => $attr->title)
                ),
                'item' => array()
            );
            if ($attr->parent_id === null) {
                $tree['item'][] = $node;
            } else {
                $parent = &self::_findNode($tree, $attr->parent_id);
                $parent['item'][] = $node;
            }
        }

        return $tree;
    }

    /**
     * Get personalisation attributes tree with user state
     *
     * @param integer $userId id uf user
     *
     * @return array
     */
    public static function getTreeWithUserState($userId)
    {
        $tree = self::getTree();
        $userPAttributes = UserPersonalisationAttribute::model()->findAllByAttributes(array('user_id' => $userId));
        foreach ($userPAttributes as $pAttr) {
            $node = &self::_findNode($tree, $pAttr->personalisation_attribute_id);
            $node['checked'] = 1;
        }
        return $tree;
    }

    /**
     * Helper method used for navigating by recursive tree
     *
     * @param array  &$node link to array branch
     * @param string $id    key to find in array
     *
     * @return array|null
     */
    private static function &_findNode(&$node, $id)
    {
        if ($node['id'] == $id) {
            return $node;
        } else {
            foreach ($node['item'] as &$item) {
                $res = &self::_findNode($item, $id);
                if ($res) {
                    return $res;
                }
            }
        }
        $empty = null;
        return $empty;
    }
}